Vows = require("vows")
assert = require("assert")
Path = require("path")
_ = require("underscore")

Eventable = require("../src/eventable")

Vows.describe("Eventables").addBatch(

	"An emitter with a listener": 
		
		topic: -> 
			
			emitter = new Eventable
			emitter.context = true
			
			emitter.on "listener", -> 
				emitter.emitted = true
				emitter.hasContext = @context
			
			emitter.emit "listener", (err) =>
				@callback(null, emitter)
				
		"fires its listener on the event": (emitter) ->
			assert.isTrue(emitter.emitted)	
		
		"runs its listeners within its own context": (emitter) ->
			assert.isTrue(emitter.hasContext)	
	
	"An emitter without listeners": 
	
		topic: ->
		
			emitter = new Eventable
			emitter.emit("noListener", @callback)
	
		"fires its listener on the event": (emitter) ->
			assert.isTrue(true)
	
	"An emitter with an listener that takes args": 
	
		topic: ->
		
			emitter = new Eventable
			emitter.on "listenerArgs", (test) ->
				emitter.emitted = test

			emitter
	
		"passes the args to the listeners and callback": (emitter) ->
			emitter.emit "listenerArgs", true, (test) ->
				assert.isTrue(emitter.emitted)
	
	"An emitter with multiple listeners": 
	
		topic: ->
		
			emitter = new Eventable
			emitter.emitted = []
		
			listener = -> 
				emitter.emitted.push(true)
			
		
			emitter.on("listeners", listener)
			emitter.on("listeners", listener)
		
			return emitter
	
		"fires its listener on the event": (emitter) ->
			emitter.emit "listeners", -> 
				assert.isTrue(emitter.emitted[0])
				assert.isTrue(emitter.emitted[1])
	
	"An emitter with async listeners": 
	
		topic: ->
		
			emitter = new Eventable
		
			emitter.on "async", (callback) ->
				setTimeout(->
					emitter.emitted = true
					callback()
				, 0)
			
			emitter
	
		"fires its listener on the event": (emitter) ->
			emitter.emit "async", -> 
				assert.isTrue(emitter.emitted)

).export(module)